# Generated by Django 4.1 on 2023-06-09 09:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("log_outgoing_requests", "0003_alter_outgoingrequestslog_options_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="hostname",
            field=models.CharField(
                help_text="The netloc/hostname part of the url.",
                max_length=255,
                verbose_name="Hostname",
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="req_body_encoding",
            field=models.CharField(
                blank=True,
                help_text="The encoding is either extracted from the Content-Type header or, if absent, taken from the default encoding configured for the content type. If the decoded request content is not displaying correctly, you may try changing the encoding value here.",
                max_length=24,
                verbose_name="Request encoding",
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="req_content_type",
            field=models.CharField(
                help_text="The content type of the request.",
                max_length=50,
                verbose_name="Request content type",
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="req_headers",
            field=models.TextField(
                help_text="The request headers.", verbose_name="Request headers"
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="res_body_encoding",
            field=models.CharField(
                blank=True,
                help_text="The encoding is either extracted from the Content-Type header or, if absent, taken from the default encoding configured for the content type. If the decoded response content is not displaying correctly, you may try changing the encoding value here.",
                max_length=24,
                verbose_name="Response encoding",
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="res_content_type",
            field=models.CharField(
                help_text="The content type of the response.",
                max_length=50,
                verbose_name="Response content type",
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="res_headers",
            field=models.TextField(
                help_text="The response headers.", verbose_name="Response headers"
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="trace",
            field=models.TextField(
                help_text="Text providing information in case of request failure.",
                verbose_name="Trace",
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="url",
            field=models.URLField(
                help_text="The url of the outgoing request.",
                max_length=1000,
                verbose_name="URL",
            ),
        ),
    ]
